@@ -1,5 +1,8 @@
 
-#define _BSD_SOURCE /* For atoll and lutimes */
+/* For atoll and lutimes */
+#define _BSD_SOURCE
+/* The new non-deprecated version of _BSD_SOURCE */
+#define _DEFAULT_SOURCE
 
 #include <config.h>
 #include <stdlib.h>
